#include <Magick++/Image.h>
#include <filesystem>
#include <fstream>

// Macro to convert 8-bit RGB components into a single 15 bit RGB triplet.
#define RGBtoRGB15(r,g,b,a) (((a) ? 0x8000 : 0) | ((r >> 3) & 0x1F) | (((g >> 3) & 0x1F) << 5) | (((b >> 3) & 0x1F) << 10))

namespace fs = std::filesystem;

void Print_Help();

int main(int argc, char* argv[])
{
  if (argc <= 1 || argc > 3) {
    Print_Help();
    return 1;
  }

  fs::path str_input = argv[1];
  fs::path str_output = "";
  std::ifstream file_in;
  std::ofstream file_out;
  Magick::Image magick_img;
  uint32_t size_width;
  uint32_t size_height;
  uint32_t *buffer_rgba;
  uint16_t *buffer_argb15;

  if (argc == 3) str_output = argv[2];
  else
  {
    printf("\nNo explicit output path. Borrowing from input path\n");
    str_output = str_input.parent_path() / str_input.stem();
    str_output += ".bin";
  }

  file_in.open(str_input, std::ios::binary);
  if (!file_in.good())
  {
    printf("\nInput file doesnt exists!\n");
    return 1;
  }
  file_in.close();

  magick_img.read(str_input);
  size_width = magick_img.columns();
  size_height = magick_img.rows();

  buffer_rgba = new uint32_t[size_width * size_height];
  buffer_argb15 = new uint16_t[size_width * size_height];

  printf("\n");
  printf("input: %s\n", str_input.c_str());
  printf("output: %s\n", str_output.c_str());
  printf("dimension: %ux%u\n", size_width, size_height);
  printf("\n");

  magick_img.write(0, 0, size_width, size_height, "RGBA", Magick::CharPixel, buffer_rgba);

  for (int i=0, n=size_width*size_height; i<n; i++)
  {
    uint32_t rgba = buffer_rgba[i];
    uint8_t a = (rgba >> 24) & 0xFF;
    uint8_t b = (rgba >> 16) & 0xFF;
    uint8_t g = (rgba >> 8) & 0xFF;
    uint8_t r = rgba & 0xFF;

    buffer_argb15[i] = RGBtoRGB15(r, g, b, a);
  }

  file_out.open(str_output, std::ios::binary);
  file_out.write(reinterpret_cast<char*>(&size_width), sizeof(uint32_t));
  file_out.write(reinterpret_cast<char*>(&size_height), sizeof(uint32_t));
  file_out.write(reinterpret_cast<char*>(buffer_argb15), size_width*size_height*sizeof(uint16_t));
  file_out.close();

  printf("\ndone...\n");

  // clean up
  delete[] buffer_rgba;
  delete[] buffer_argb15;
  return 0;
}

void Print_Help()
{
  printf("\n");
  printf("Simple image converter into raw argb15 color for DS %s\n\n", VERSION);
  printf("Usage: sillyimage <input image> <output image>\n");
  printf("\n");
}